Attribute VB_Name = "comsubs"
' Definitions of offsets within the CICS Commarea for VERIFYPW
Private Const ofs_ver      As Integer = 0
Private Const ofs_resp     As Integer = 2
Private Const ofs_user     As Integer = 12
Private Const ofs_password As Integer = 20
Private Const ofs_client   As Integer = 28
Private Const len_ver      As Integer = 2
Private Const len_resp     As Integer = 5
Private Const len_user     As Integer = 8
Private Const len_password As Integer = 8
Private Const len_client   As Integer = 4

' Current version of CICS VERIFYPW COMMAREA
Private Const verify_ver As String = "00" * 2
Private Const verifypw_prog As String = "VERIFYPW"

' Length of CICS COMMAREA for VERIFYPW
Private Const verify_calen As Integer = 100

' location of CICSCLI.INI file
Private Const cicscli = "c:\cicscli\bin\cicscli.ini"

' Offsets and lengths in the standard header
Private Const ofs_version = 0
Private Const ofs_trnid = 4
Private Const ofs_trmid = 8
Private Const ofs_nxtrnid = 12
Private Const ofs_mapname = 16
Private Const ofs_maplen = 40
Private Const ofs_calen = 48
Private Const len_version = 4
Private Const len_trnid = 4
Private Const len_trmid = 4
Private Const len_nxtrnid = 4
Private Const len_mapname = 8
Private Const len_maplen = 4
Private Const len_calen = 5

' Offsets and lengths for SEND TEXT
Public Const ofs_sendtext = 80
Public Const len_sendtext = 100

' Variable used to remember the CICS client name
Public client_name As String * 4
Public ca_user As String * 8
Dim ca_pw As String * 8
Public ca_system As String * 8

' Public variables to return error messages to the forms
Public cics_abcode As String
Public cics_errmsg As String

' Pointers to the CICS client objects
Public ECI As Object
Private Connect As Object
Private Flow As Object
Private buffer As Object
Private UOW As Object

' Enumeration for data types
Public Enum comm_data_type
 is_number = 1
 is_char = 0
End Enum

' Total length of all the BMS fields in the record
Public BMS_Length As Integer

' subroutine to create the CICS Client ECI objects
Public Sub createECIobjects()
    ' Create the single ECI object
    Set ECI = CreateObject("Ccl.ECI")

    ' Turn off the error pop up - we will check the ExCode for errors
    ECI.ErrorWindow (False)

    ' Create connection and flow objects
    Set Connect = CreateObject("Ccl.Connect")
    Set Flow = CreateObject("Ccl.Flow")
 
    ' Allocate a buffer object to hold the CICS COMMAREA
    Set buffer = CreateObject("Ccl.Buffer")
 
    ' We don't need a unit of work object
    Set UOW = Nothing
End Sub

' subroutine to delete the CICS Client ECI objects
Public Sub deleteECIobjects()
    ' Delete the ECI objects we created
    Set buffer = Nothing
    Set Flow = Nothing
    Set Connect = Nothing
    Set ECI = Nothing
End Sub

' subroutine to right justify and pad on left with zeros
' input parameters are a string and a desired length
' this routine is aimed at filling in data destined for a
' CICS COBOL program
Private Sub padzeros(st As String, initlen As Integer)
    ' trim off any trailing blanks
    Trim (st)
 
    If Left(st, 1) = " " Then
    Mid$(st, 1) = "0"
    End If
 
    ' Insert enough leading zeros to get back to where we were
    While Len(st) < initlen
        st = "0" + st
    Wend
End Sub

Public Sub to_int(ca() As Byte, in_int As Integer, ca_ofs As Integer)
    Dim templong As Long
    
    ' We need to convert a negative number by adding 65536
    ' This is due to the fact that a byte holds numbers from 0 to 255
    ' and thus cannot hold a negative number
    If in_int < 0 Then
        templong = 65536
    Else
        templong = 0
    End If
    
    'Get the input variable
    templong = templong + in_int
    
    ca(ca_ofs) = templong Mod 256
    ca(ca_ofs + 1) = (templong \ 256) Mod 256
End Sub

Public Sub to_long(ca() As Byte, in_long As Long, ca_ofs As Integer)
    Dim i As Integer
    Dim templong As Long
    If in_long < 0 Then
        templong = in_long + 2147483647#
        templong = templong + 1
    Else
        templong = in_long
    End If
    ca(ca_ofs) = templong Mod 256
    ca(ca_ofs + 1) = (templong \ 256) Mod 256
    ca(ca_ofs + 2) = (templong \ 65536) Mod 256
    ca(ca_ofs + 3) = (templong \ 16777216) Mod 256
    
    If in_long < 0 Then
        ca(ca_ofs + 3) = ca(ofs + 3) + 128
    End If
    
    'ca(ca_ofs) = in_long Mod 256
    'ca(ca_ofs + 1) = (in_long \ 256) Mod 256
    'ca(ca_ofs + 2) = (in_long \ 65536) Mod 256
    'ca(ca_ofs + 3) = (in_long \ 16777216) Mod 256
End Sub

Public Function from_int(ca() As Byte, ca_ofs As Integer) As Integer
      from_int = ca(ca_ofs) + (ca(ca_ofs + 1) * 256)
End Function

Public Function from_long(ca() As Byte, ca_ofs As Integer) As Long
    from_long = ca(ca_ofs) + (ca(ca_ofs + 1) * CLng(256)) + (ca(ca_ofs + 2) * 65536) + (ca(ca_ofs + 3) * 16777216)
End Function

' This routine moves data into the CICS COMMAREA byte array (ca).  The data is
'  padded on the right with spaces or on the left with zeros depending on the
' type of data (indicated by the value of dtype).
Public Sub to_comm(ca() As Byte, in_str As String, ca_ofs As Integer, ca_len As Integer, dtype As comm_data_type)
    Dim st As String
    
    ' Validate the input parameters
    Debug.Assert ca_len > 0
    Debug.Assert ca_ofs >= 0
    
    ' Get the input string
    st = in_str
    
    ' pad the string to the proper length
    If dtype = is_number Then
        padzeros st, ca_len
    Else
        While Len(st) < ca_len
            st = st + " "
        Wend
    End If
    
    ' Make sure that the result is not too long
    If Len(st) > ca_len Then
        st = Left(st, ca_len)
    End If
    
    ' Now, move the data into the COMMAREA array
    For i = 1 To ca_len
        ca(ca_ofs + i - 1) = Asc(Mid(st, i, 1))
    Next
End Sub

' This subroutine inserts data into the CICS COMMAREA and also sets a BMS length
' field with the length of the input data.
Public Sub move_BMS(ca() As Byte, in_str As String, ca_ofs As Integer, ca_len As Integer, dtype As comm_data_type, ca_BMS_ofs As Integer)
    ' Validate the input parameters
    Debug.Assert ca_len > 0
    ' Note = we do not allow a BMS field in the first position of the COMMAREA
    Debug.Assert ca_ofs > 0
    Debug.Assert ca_ofs = ca_BMS_ofs + 3
    
    ' move the data to the BMS input area
    to_comm ca, in_str, ca_ofs, ca_len, dtype
    
    ' fill in the BMS length field
    to_int ca, Len(in_str), ca_BMS_ofs
    
    ' Keep track of the total length of all fields
    BMS_Length = BMS_Length + Len(in_str)
End Sub

' This function is used to extract data from the COMMAREA and return
' the result as a string
Public Function from_comm(ca() As Byte, ByVal ca_ofs As Integer, ByVal ca_len As Integer) As String
    ' result string
    Dim st As String
    
    ' work variable
    Dim i As Integer
    
    ' Validate the input parameters
    Debug.Assert ca_len > 0
    Debug.Assert ca_ofs >= 0
    
    ' set the work variable to the location of the first byte within the COMMAREA
    i = ca_ofs
    
    ' initialize the string to a null string
    st = ""
    
    ' build the result string
    While Len(st) < ca_len
        st = st + Chr(ca(i))
        i = i + 1
    Wend
    
    ' strip trailing blanks
    RTrim (st)
    
    ' Return the result
    from_comm = st
End Function

' subroutine to set the standard header part of the CICS COMMAREA
Public Sub SetStandardHeader(ca() As Byte, cics_tran As String, map As String, calen As Integer)
    Dim maplength As String
    Dim calength As String
 
    ' get the data into strings of the proper length, padding
    ' character data with spaces on the right and number data with
    ' zeros on the left
    maplength = Str(BMS_Length)
    calength = Str(calen)
 
    ' Set the fields in the COMMAREA
    to_comm ca, "0000", ofs_version, len_version, is_number
    to_comm ca, cics_tran, ofs_trnid, len_trnid, is_char
    to_comm ca, client_name, ofs_trmid, len_trmid, is_char
    to_comm ca, map, ofs_mapname, len_mapname, is_char
    to_comm ca, maplength, ofs_maplen, len_maplen, is_number
    to_comm ca, calength, ofs_calen, len_calen, is_number
End Sub

' This subroutine extracts the CICS map name from the Standard header
' It is used to determine which map was used for the returned data
Public Function getmapname(ca() As Byte) As String
    Dim st As String

    ' Get the map name from the standard header in the COMMAREA
    getmapname = from_comm(ca, ofs_mapname, len_mapname)
End Function

' Subroutine to execute the CICS program and return the ECI return code to
' the calling program.  If the call is successful, the COMMAREA passed in
' is updated.
Public Function executeCICSTran(ca() As Byte, pgmname As String, commlen As Integer) As Integer
    Dim i As Integer
    Dim rc As Integer
    Dim prog As String * 8
    Dim tempca As Variant
    
    ' Set the error message and abend code to nulls
    cics_errmsg = ""
    cics_abcode = ""
            
    ' force the program name to be 8 bytes padded with spaces
    prog = pgmname

    ' move the COMMAREA data into the buffer object
    buffer.SetData ca
    
    ' Execute the CICS NEWCALL program
    On Error Resume Next
    Connect.Link Flow, prog, buffer, UOW
    
    ' don"t trap other errors
    On Error GoTo 0

    ' Get the return code
    rc = ECI.ExCode
    
    ' Check the return code
    If rc = cclNoError Then
        ' restore the data to the byte array
        tempca = buffer.Data

        ' Get the data out of the Variant into the original byte array
        For i = 0 To commlen - 1
            ca(i) = tempca(i)
        Next i
    Else
        ' get the CICS error messages and abend code, if present
        cics_errmsg = ECI.ExCodeText
        cics_abcode = Flow.AbendCode

        ' The following 3 statements are to code around a bug in the
        ' client exception handling routines, where the exception
        ' code is not being reset on subsequent calls
        Set ECI = Nothing
        Set ECI = CreateObject("Ccl.ECI")
        ECI.ErrorWindow (False)
    End If
    
    ' set the return code
    executeCICSTran = rc
End Function
Public Function verify_password(cics_user As String, cics_pw As String, cics_system As String, cics_client As String) As Integer
    Dim i As Integer
    Dim rc As Integer
    Dim ca_rc As String * 5
    Dim ca(100) As Byte
    Dim tempca As Variant
 
    ' Set the length of the COMMAREA for VERIFYPW
    buffer.SetLength verify_calen
 
    ' Get the userid, password and client name in fixed length strings
    ca_user = cics_user
    ca_pw = cics_pw
    ca_system = cics_system
    client_name = cics_client
 
    ' clear the commarea data to nulls
    For i = 0 To 99
        ca(i) = 0
    Next i
 
    ' Set the Version Number, Userid, Password and client name in the input COMMAREA
    to_comm ca, verify_ver, ofs_ver, len_ver, is_number
    to_comm ca, ca_user, ofs_user, len_user, is_char
    to_comm ca, ca_pw, ofs_password, len_password, is_char
    to_comm ca, client_name, ofs_client, len_client, is_char
 
    ' Set the CICS region, and use a hard coded userid and password
    ' This is because we do not yet know if the userid and password
    ' provided are in fact valid.
    Connect.Details ca_system, ca_user, ca_pw
 
    ' Execute the CICS VERIFYPW program
    rc = executeCICSTran(ca, verifypw_prog, verify_calen)
    
    ' Check if the ECI call worked
    If rc = cclNoError Then
        ' Get the Client Name from the COMMAREA
        client_name = from_comm(ca, ofs_client, len_client)
        
        ' Get the return code from the verify password
        ca_rc = from_comm(ca, ofs_resp, len_resp)
  
        ' Convert it to an integer
        rc = Val(ca_rc)

        ' Check if the user id and password are valid
        If rc <> 0 Then
            If rc = 69 Then
                cics_errmsg = "Invalid User id"
            ElseIf rc = 70 Then
                cics_errmsg = "Invalid Password"
            Else
                cics_errmsg = "Invalid User id or Password"
            End If
        End If
    ElseIf rc = cclTransaction And cics_abcode = "AEY7" Then
        cics_errmsg = "Invalid User id or Password"
        cics_abcode = vbNullString
    End If
 
    ' Discard the buffer object
    verify_password = rc
End Function

' Routine to try and find the CICS Client name in the CICSCLI.INI file
' If the client name is found, then it will be returned.  If the client
' is an asterisk ("*") or is not found, then a blank name will be returned
Public Function getclientname() As String
    Dim clname As String * 4
    Dim st As String
    Dim fileline As String
    Dim foundname As Boolean
    Dim i As Integer
    
    ' initialize the client name to blanks
    clname = "    "
    
    ' initialize the boolean value to indicate if we have found a name
    foundname = False
    
    ' trap file errors
    On Error GoTo badfile
    
    ' try to open the CICSCLI.INI file
    Open cicscli For Input As #1
    
    ' Read the file, looking for the CLIENT =
    Do Until foundname Or EOF(1)
        ' read a line of input
        Line Input #1, fileline
        
        ' remove leading and trailing blanks
        Trim (fileline)
        
        ' make sure that the line has data and is not a comment
        If Len(fileline) > 7 And Left(fileline, 1) <> ";" Then
            ' First, try to find the word CLIENT
            i = InStr(1, fileline, "CLIENT", vbTextCompare)
            If i > 0 Then
                ' try to find an equal sign ("=")
                i = InStr(i, fileline, "=")
                If i > 0 Then
                    ' skip the equal sign
                    i = i + 1
                    
                    ' skip any blanks in front of the client name
                    While i < Len(fileline) And Mid(fileline, i, 1) = " "
                        i = i + 1
                    Wend
                    ' make sure we are not at the end of the line
                    If i < Len(fileline) Then
                        'Make sure it is not an asterisk (*)
                        If Mid(fileline, i, 1) <> "*" Then
                            ' Get the client name
                            clname = Mid(fileline, i, 4)
                        End If
                        
                        ' Break the loop
                        foundname = True
                    End If
                End If
            End If
        End If
    Loop
    
    ' close the CICSCLI.INI file
    Close #1
    
badfile:
    ' check if we found a client name
    If foundname Then
        ' return the client name
        st = clname
    Else
        ' return a null string
        st = vbNullString
    End If
    
    ' return the result as a string
    getclientname = st
End Function
